/*      $Id: bnews-speed.c,v 1.2 2002/07/28 15:01:56 stef Exp $ */

/* 
    Copyright (C) 2001,2002 Stef Van Dessel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "bnews.h"

int main(int argc, char *argv[])
{
    long encodecount = 1;
    long decodecount = 1;
    long long timeinterval;
    int verbose = 0;
    int i, j;
    long k;
    struct timeval *tv1, *tv2, *delta;
    struct timezone *tz;

    char c;
    char in[40], rev[40], out[41];

    while ((c = getopt(argc, argv, "d:e:v")) != EOF) {
	switch (c) {
	case 'e':
	    if (!sscanf(optarg, "%ld", &encodecount)) {
		fprintf(stderr, "Can't parse encodecount: %s\n", optarg);
		exit(6);
	    }
	    break;
	case 'd':
	    if (!sscanf(optarg, "%ld", &decodecount)) {
		fprintf(stderr, "Can't parse decodecount: %s\n", optarg);
		exit(6);
	    }
	    break;
	case 'v':
	    verbose++;
	    break;
	default:
	    break;
	}
    }

    tv1 = malloc(sizeof(struct timeval));
    tv2 = malloc(sizeof(struct timeval));
    delta = malloc(sizeof(struct timeval));
    tz = malloc(128);

    strncpy(in, "0123456789012345678901234567890123456789", 40);

    if (gettimeofday(tv1, tz) != 0) {
	fprintf(stderr, "Cannot get time of day\n");
	exit(1);
    }

    for (i = 0; i < encodecount; i++) {
	EnCode(in, out, 40);
    }

    for (i = 0; i < decodecount; i++) {
	DeCode(out, rev, 40);
    }

    if (gettimeofday(tv2, tz) != 0) {
	fprintf(stderr, "Cannot get time of day\n");
	exit(1);
    }
    delta->tv_sec = tv2->tv_sec - tv1->tv_sec - 1;
    delta->tv_usec = (1000000 - tv1->tv_usec) + tv2->tv_usec;
    while (delta->tv_usec >= 1000000) {
	delta->tv_sec++;
	delta->tv_usec -= 1000000;
    };

    timeinterval = (long long ) delta->tv_usec + ((long long) delta->tv_sec * 1000000);

    if (verbose) {
	printf("Average bandwidth = %.3lf MB/s\n", ((encodecount + decodecount) * 40) / ((float) timeinterval / 1000000) / (1024 * 1024));
	// printf("Time delay: %d.%06d\n", delta->tv_sec, delta->tv_usec);
    }

    if (memcmp(in, rev, 40)) {
	fprintf(stderr, "memcpy failed, something is wrong\n");
    }

    return 0;
}
