/*      $Id: decoder.c,v 1.22 2002/09/29 19:59:56 stef Exp $ */

/* 
    Copyright (C) 2001,2002 Stef Van Dessel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "bnews.h"

char fname[PATHSIZE + 1];
long opt_s;
char buf[2048];

unsigned char bindata[DATASIZE];



void do_help()
{
    fprintf(stderr,
	    "bnews-decoder - a bnews-decoder for POSIX compliant systems\n");
    fprintf(stderr, "(c) 2001,2002 by Stef Van Dessel\n");
    fprintf(stderr, "This software is licensed under the GPL\n");
    fprintf(stderr, "part of " PACKAGE " " VERSION "\n");
    fprintf(stderr, "Usage:\n");
    fprintf(stderr, "bnews-decoder [options]\n");
    fprintf(stderr, " -i input file : use specified input file\n");
    fprintf(stderr, " -o output file : use specified output file\n");
    fprintf(stderr, " -s size : size of encoded file\n");
    fprintf(stderr,
	    " -l lines : show summary line every l lines (verbose only)\n");
    fprintf(stderr, " -n : encoded file has no header - (specify -s!)\n");
    fprintf(stderr, " -m : encoded file has a header (default)\n");
    fprintf(stderr, " -q : be quiet\n");
    fprintf(stderr, " -v : be verbose\n");
    fprintf(stderr,
	    " -u : unlink input file on successfull decode (use with -i)\n");
    fprintf(stderr,
	    " -k : keep broken output file on failure during decode\n");
    fprintf(stderr, " -d : overwrite existing output file\n");
    fprintf(stderr,
	    " -t : send output to stdout instead of filename in header\n");
    fprintf(stderr, " -e : use filename in embedded stream (default)\n");
    fprintf(stderr, " -h : show this help\n");

    exit(9);
}

/*
// The main routine
// Wait for start of data, then decode until finished.
*/

int main(int argc, char *argv[])
{
    int gfn, i;
    FILE *fp, *in;
    unsigned char *p, *q;
    int c, r, s;
    long f, b = 0;
    unsigned char *out;
    extern int optind;
    extern char *optarg;
    char *opt_i = NULL, *opt_o = NULL;
    int lines = 0;
    int fildes;
    int result;
    int opt_d = 0;
    int opt_e = 1;
    int opt_m = 1;
    int opt_k = 0;
    int opt_l = 50;
    int opt_v = 0;
    int opt_u = 0;



    while ((c = getopt(argc, argv, "i:o:nmtqhudvkes:l:")) != EOF) {
	switch (c) {
	case 'i':
	    opt_i = optarg;
	    break;
	case 'o':
	    opt_o = optarg;
	    break;
	case 'n':
	    opt_m = 0;
	    break;
	case 'm':
	    opt_m = 1;
	    break;
	case 'q':
	    opt_v = 0;
	    break;
	case 'e':
	    opt_e = 1;
	    break;
	case 't':
	    opt_e = 0;
	    break;
	case 'u':
	    opt_u = 1;
	    break;
	case 'd':
	    opt_d = 1;
	    break;
	case 'v':
	    opt_v = 1;
	    break;
	case 'k':
	    opt_k = 1;
	    break;
	case 'h':
	    do_help();
	    break;
	case 's':
	    if (!sscanf(optarg, "%ld", &opt_s)) {
		fprintf(stderr, "Can't parse size: %s\n", optarg);
		exit(6);
	    }
	    break;
	case 'l':
	    if (!sscanf(optarg, "%i", &opt_l)) {
		fprintf(stderr, "Can't parse size: %s\n", optarg);
		exit(6);
	    }
	    break;
	default:
	    do_help();
	    break;
	}
    }

    if (opt_i) {
	if (!(in = fopen(opt_i, "r"))) {
	    fprintf(stderr, "Failed to open input file.\n");
	    exit(5);
	}
    } else {
	in = stdin;
    }

    if (opt_m) {
	do {
	    if (fgets(buf, sizeof(buf), in) != NULL)
		gfn = GetFileName(buf, fname, &opt_s);
	}
	while ((gfn == 0) && !feof(in));
    }

    if (opt_o) {
	strncpy(fname, opt_o, sizeof(fname));
    };

    if (feof(in)) {
	fprintf(stderr,
		"Filename specification not found in input stream.\n");
	exit(1);
    }

    fp = stdout;
    if (opt_e) {
	/* do not go through fopen, to prevent overwriting files */
	if (opt_d) {
	    fildes = open(fname, O_CREAT | O_WRONLY, 0666);
	} else {
	    fildes = open(fname, O_CREAT | O_WRONLY | O_EXCL, 0666);
	}
	if (fildes > 0) {
	    fp = fdopen(fildes, "wb");
	    if (fp == NULL) {
		fprintf(stderr,
			"ERROR: Failed to create destination file.\n");
		exit(5);
	    }
	} else {
	    fprintf(stderr, "ERROR: Destination file already exists.\n");
	    exit(6);
	}

    }

    buf[0] = '.';
    f = opt_s - DATASIZE;
    // read at buf + 1 to easy dot escaping  - thx to carl

    do {
	q = (unsigned char *) buf + 1;
	if (fgets(q, 1024, in) != NULL) {
	    lines++;
	    // split in 41 byte packets, decode & write

	    for (p = q, i = 0; *p >= 32; p++);

	    i = (int) (p - q);
	    if (i > 255) {
		fprintf(stderr, "Error: Broken input line, bailing out");
		if ((fp != stdout) && (opt_k != 0)) {
		    unlink(fname);
		}
		return (3);
	    }

	    r = i / CODESIZE;
	    s = i % CODESIZE;

	    if (s) {
		if (s == CODESIZE - 1) {
		    // fprintf(stderr, "Unescaping dot r=%d, s=%d, i=%d\n", r, s, i);
		    q = (unsigned char *) buf;
		} else {
		    // THIS IS NOW AN ERROR ~
		    fprintf(stderr,
			    "ERROR: Not fixing up line (rest=%d)\n", s);
		    if ((fp != stdout) && (opt_k != 0)) {
			unlink(fname);
		    }
		    return (7);
		    /*
		       for (; p < q + (CODESIZE * (r + 1)); p++) {
		       *p = ' ';
		       }
		     */
		}
		r++;
		s = 0;
	    }
	    // a simple sanity check to make sure we haven't smashed our stack/heap
	    if (f + DATASIZE != opt_s) {
		fprintf(stderr, "ERROR: opt_s=%ld, b=%ld, f=%ld\n", opt_s,
			b, f);
		if ((fp != stdout) && (opt_k != 0)) {
		    unlink(fname);
		}
		return (2);
	    }
	    while (r--) {
		result = DeCode(q, bindata, DATASIZE);
		if (b < f) {
		    b += fwrite(bindata, 1, DATASIZE, fp);
		} else {
		    b += fwrite(bindata, 1, (int) (opt_s - b), fp);
		}

		q += CODESIZE;
	    }
	    if (opt_v && (lines % opt_l == 0)) {
		fprintf(stdout, "Status: %010ld / %010ld bytes %s\r", b,
			opt_s, fname);
	    }
	}
    }
    while (feof(in) == 0);
    if (opt_v) {
	fprintf(stdout, "Status: %010ld / %010ld bytes %s\r\n", b, opt_s,
		fname);
    }
    fclose(fp);

    if (b != opt_s) {
	fprintf(stderr, "ERROR: opt_s=%ld, b=%ld\n", opt_s, b);
	if ((fp != stdout) && (opt_k != 0)) {
	    unlink(fname);
	}
	return (1);
    }
    if (opt_u && opt_i) {
	unlink(opt_i);
    };

    return 0;
}
